using System;
using UnityEngine;
using System.Collections.Generic;

namespace Vinput
{
    public class WindowCamAligner : MVRScript
    {
        private Atom _windowCamera;
        private Transform _navTransform;
        private JSONStorableStringChooser _hotkeyJSON;
        private KeyCode _hotkey = KeyCode.Comma;

        public static List<string> _hotkeys = Enum.GetNames(typeof(KeyCode)).ToList();

        public override void Init()
        {
            if (containingAtom.type != "WindowCamera")
            {
                SuperController.LogError($"{nameof(WindowCamAligner)}: Plugin must be attached to the WindowCamera.");
                return;
            }

            _navTransform = SuperController.singleton.centerCameraTarget.transform;

            SetupUI();

        }

        public void Update()
        {
            if (Input.GetKeyDown(_hotkey)) Align();
        }

        private void Align()
        {
            // SuperController.LogMessage($"{_navTransform.position}, {_navTransform.rotation.eulerAngles}");

            containingAtom.freeControllers[0].currentPositionState = FreeControllerV3.PositionState.Lock;
            containingAtom.freeControllers[0].currentRotationState = FreeControllerV3.RotationState.Lock;

            containingAtom.freeControllers[0].transform.position = _navTransform.position;
            containingAtom.freeControllers[0].transform.rotation = _navTransform.rotation;
            containingAtom.freeControllers[0].transform.localPosition -= containingAtom.freeControllers[0].transform.forward * 0.15f;

            // SuperController.LogMessage($"{containingAtom}: {containingAtom.transform.position}, {containingAtom.transform.rotation.eulerAngles}");
        }

        private void OnHotkeyChanged(string newKey)
        {
            _hotkey = (KeyCode)Enum.Parse(typeof(KeyCode), newKey);
        }

        private void SetupUI()
        {
            UIDynamicButton btn = CreateButton("Align");
            btn.button.onClick.AddListener(Align);

            _hotkeyJSON = new JSONStorableStringChooser("Shortcut Key", _hotkeys, "Comma", "Shortcut Key");
            RegisterStringChooser(_hotkeyJSON);
            _hotkeyJSON.setCallbackFunction += OnHotkeyChanged;
            CreateScrollablePopup(_hotkeyJSON);

            CreateTextField(new JSONStorableString("text", "Press <b>Align</b> or the shortcut key to align WindowCamera to player view.\n\nNote: Views will not be the same unless WindowCamera FOV is the same as the player FOV."), true).height = 200;

        }
    }
}